﻿using GarmentFactoryContracts.BindingModels;
using GarmentFactoryContracts.BusinessLogicsContracts;
using GarmentFactoryContracts.SearchModels;
using GarmentFactoryContracts.StoragesContracts;
using GarmentFactoryContracts.ViewModels;
using GarmentFactoryDataModels.Models;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace GarmentFactoryBusinessLogic
{
    public class ShopLogic : IShopLogic
    {
        private readonly ILogger _logger;

        // Хранилище всех магазинов
        private readonly IShopStorage _shopStorage;

        private readonly ITextileStorage _textileStorage;

        public ShopLogic(ILogger<ShopLogic> logger, IShopStorage shopStorage, ITextileStorage textileStorage)
        {
            _logger = logger;
            _shopStorage = shopStorage;
            _textileStorage = textileStorage;
        }

        // Чтение конкретного магазина
        public ShopViewModel? ReadElement(ShopSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }

            _logger.LogInformation("ReadElement. ShopName:{ShopName}. Id:{Id}", model.ShopName, model.Id);
            var element = _shopStorage.GetElement(model);

            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");

                return null;
            }

            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);

            return element;
        }

        // Чтение всех магазинов
        public List<ShopViewModel>? ReadList(ShopSearchModel? model)
        {
            _logger.LogInformation("ReadList. ShopName:{ShopName}. Id: {Id}", model?.ShopName, model?.Id);

            var list = model == null ? _shopStorage.GetFullList() : _shopStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        // Создание магазина
        public bool Create(ShopBindingModel model)
        {
            CheckModel(model);

            if (_shopStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }

        // Обновление магазина
        public bool Update(ShopBindingModel model)
        {
            CheckModel(model);

            if (_shopStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }

        // Удаление магазина
        public bool Delete(ShopBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);

            if (_shopStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");

                return false;
            }
            return true;
        }

        // Пополнение магазина
        public bool MakeSupply(SupplyBindingModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }

            if (model.Count <= 0)
            {
                throw new ArgumentNullException("Количество добавляемых изделий должно быть больше 0");
            }

			var textile = _textileStorage.GetElement(new TextileSearchModel { Id = model.TextileId });
			if (textile == null)
			{
				throw new ArgumentException($"При добавлении товара в магазин товар с id={model.TextileId} не найден");
			}

			var shop = _shopStorage.GetElement(new ShopSearchModel { Id = model.ShopId });
			if (shop == null)
			{
				throw new ArgumentException($"При добавлении товара в магазин магазин c id={model.ShopId} не найден");
			}

            int countTextilesInShop = shop.ShopTextiles.Select(x => x.Value.Item2).Sum();

            //Если в текущий магазин помещается столько товаров
            if (countTextilesInShop + model.Count <= shop.TextileMaxCount)
            {
				// Если такой товар есть, то прибавление переданного кол-ва
				if (shop.ShopTextiles.ContainsKey(model.TextileId))
				{
					var oldValue = shop.ShopTextiles[model.TextileId];
					oldValue.Item2 += model.Count;
					shop.ShopTextiles[model.TextileId] = oldValue;
				}
				// Если такого товара нет, то кол-во такого товара равно переданному кол-ву
				else
				{
					shop.ShopTextiles.Add(model.TextileId, (textile, model.Count));
				}

                _shopStorage.Update(new()
                {
                    Id = shop.Id,
                    ShopName = shop.ShopName,
                    Address = shop.Address,
                    DateOpen = shop.DateOpen,
                    ShopTextiles = shop.ShopTextiles,
                    TextileMaxCount = shop.TextileMaxCount,
                });
				return true;
			}
            //Если не поместилось столько товара
            _logger.LogWarning("Required shop is overflowed");
            return false;	
        }

        // Проверка данных магазина при добавлении/удалении/обновлении
        private void CheckModel(ShopBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }

            // При удалении проверка дальше не нужна
            if (!withParams)
            {
                return;
            }

            // Проверка наличия названия
            if (string.IsNullOrEmpty(model.ShopName))
            {
                throw new ArgumentNullException("Отсутствует название магазина", nameof(model.ShopName));
            }

            // Проверка наличия адреса
            if (string.IsNullOrEmpty(model.Address))
            {
                throw new ArgumentNullException("Отсутствует адреса магазина", nameof(model.Address));
            }

            _logger.LogInformation("Shop. ShopName:{ShopName}. Address:{Address}. Id: {Id}", model.ShopName, model.Address, model.Id);

            var element = _shopStorage.GetElement(new ShopSearchModel { ShopName = model.ShopName });

            if (element != null && element.Id != model.Id)
            {
                throw new InvalidOperationException("Магазин с таким названием уже есть");
            }
        }

		public bool Sell(SupplySearchModel model)
		{
			if (!model.TextileId.HasValue || !model.Count.HasValue)
			{
				return false;
			}
			if (_shopStorage.Sell(model))
			{
				_logger.LogInformation("Selling sucsess");
				return true;
			}
			_logger.LogInformation("Selling failed");
			return false;
		}
	}
}
