﻿using GarmentFactoryContracts.BindingModels;
using GarmentFactoryContracts.SearchModels;
using GarmentFactoryContracts.StoragesContracts;
using GarmentFactoryContracts.ViewModels;
using GarmentFactoryFileImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;

namespace GarmentFactoryFileImplement.Implements
{
	public class ShopStorage : IShopStorage
	{
		private readonly DataFileSingleton source;

		public ShopStorage()
		{
			source = DataFileSingleton.GetInstance();
		}

		public List<ShopViewModel> GetFullList()
		{
			return source.Shops.Select(x => x.GetViewModel).ToList();
		}

		public List<ShopViewModel> GetFilteredList(ShopSearchModel model)
		{
			if (string.IsNullOrEmpty(model.ShopName))
			{
				return new();
			}
			return source.Shops.Where(x => x.ShopName.Contains(model.ShopName)).Select(x => x.GetViewModel).ToList();
		}

		public ShopViewModel? GetElement(ShopSearchModel model)
		{
			if (string.IsNullOrEmpty(model.ShopName) && !model.Id.HasValue)
			{
				return null;
			}
			return source.Shops.FirstOrDefault(x =>
			(!string.IsNullOrEmpty(model.ShopName) && x.ShopName == model.ShopName) ||
			(model.Id.HasValue && x.Id == model.Id))?.GetViewModel;
		}

		public ShopViewModel? Insert(ShopBindingModel model)
		{
			model.Id = source.Shops.Count > 0 ? source.Shops.Max(x => x.Id) + 1 : 1;
			var newShop = Shop.Create(model);
			if (newShop == null)
			{
				return null;
			}
			source.Shops.Add(newShop);
			source.SaveShops();
			return newShop.GetViewModel;
		}

		public ShopViewModel? Update(ShopBindingModel model)
		{
			var shop = source.Shops.FirstOrDefault(x => x.Id == model.Id);
			if (shop == null)
			{
				return null;
			}
			shop.Update(model);
			source.SaveShops();
			return shop.GetViewModel;
		}

		public ShopViewModel? Delete(ShopBindingModel model)
		{
			var shop = source.Shops.FirstOrDefault(x => x.Id == model.Id);
			if (shop != null)
			{
				source.Shops.Remove(shop);
				source.SaveShops();
				return shop.GetViewModel;
			}
			return null;
		}

		public bool Sell(SupplySearchModel model)
		{
			if (model == null || !model.TextileId.HasValue || !model.Count.HasValue)
			{
				return false;
			}

			//магазины по степени наполненности (от самого наполненного) с этим текстилем
			var shopsWithThisTextile = source.Shops.Where(x => x.Textiles.ContainsKey(model.TextileId.Value)).OrderByDescending(x => x.Textiles[model.TextileId.Value]).ToList();

			int thisTextileInShops = shopsWithThisTextile.Select(x => x.Textiles[model.TextileId.Value]).Sum();

			if (thisTextileInShops < model.Count)
			{
				return false;
			}

			foreach (var shop in shopsWithThisTextile)
			{
				//кол-во необходимых текстилей после возможной продажи всех текстилей
				int thisTextileNeed = model.Count.Value - shop.Textiles[model.TextileId.Value];

				//Если текстиль ещё нужен будет
				if (thisTextileNeed > 0)
				{
					//удаление всех текстилей в этом магазине
					shop.Textiles.Remove(model.TextileId.Value);
					shop.TextilesUpdate();
					model.Count = thisTextileNeed;
				}

				else
				{
					//Если ровно хватает
					if (thisTextileNeed == 0)
					{
						shop.Textiles.Remove(model.TextileId.Value);
					}
					//Если нужно меньше, чем есть в магазине
					else
					{
						//уменьшение кол-ва
						shop.Textiles[model.TextileId.Value] = -thisTextileNeed;
					}
					shop.TextilesUpdate();
					source.SaveShops();
					return true;
				}
			}
			source.SaveShops();
			return false;
		}

		public bool RestockShops(SupplyBindingModel model)
		{
			//подсчёт свободных мест через сумму свободных мест в каждом магазине
			//в каждом магазине кол-во свободных мест = макс. кол-во - занятые места
			int totalFreeSpace = source.Shops
				.Select(x => x.TextileMaxCount - x.ShopTextiles.Select(y => y.Value.Item2).Sum())
				.Sum();

			//если места не хватает
			if (totalFreeSpace < model.Count)
			{
				return false;
			}

			var shopsWithFreeSpace = source.Shops.Where(x => x.TextileMaxCount - x.ShopTextiles.Select(x => x.Value.Item2).Sum() > 0);

			foreach (Shop shop in shopsWithFreeSpace)
			{
				int freeSpace = shop.TextileMaxCount - shop.ShopTextiles.Select(x => x.Value.Item2).Sum();

				freeSpace = Math.Min(freeSpace, model.Count);
				model.Count -= freeSpace;

				if (shop.Textiles.ContainsKey(model.TextileId))
				{
					shop.Textiles[model.TextileId] += freeSpace;
				}
					
				else
				{
					shop.Textiles.Add(model.TextileId, freeSpace);
				}

				shop.TextilesUpdate();

				if (model.Count == 0)
				{
					source.SaveShops();
					return true;
				}
			}
			return false;
		}
	}
}
